@@ -55,9 +55,7 @@ static void message_set_reply_to (GMimeMessage *message, const char *reply_to);
 static void message_add_recipients_from_string (GMimeMessage *message, char *type, const char *string);
 static void message_set_subject (GMimeMessage *message, const char *subject);
 
-static ssize_t write_addrspec (GMimeStream *stream, const char *name, const char *value);
 static ssize_t write_received (GMimeStream *stream, const char *name, const char *value);
-static ssize_t write_subject (GMimeStream *stream, const char *name, const char *value);
 static ssize_t write_msgid (GMimeStream *stream, const char *name, const char *value);
 
 
@@ -134,22 +132,8 @@ g_mime_message_init (GMimeMessage *message, GMimeMessageClass *klass)
 	message->message_id = NULL;
 	message->mime_part = NULL;
 	
-	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Sender", write_addrspec);
-	g_mime_header_register_writer (((GMimeObject *) message)->headers, "From", write_addrspec);
-	g_mime_header_register_writer (((GMimeObject *) message)->headers, "To", write_addrspec);
-	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Cc", write_addrspec);
-	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Bcc", write_addrspec);
-	
-	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Resent-Sender", write_addrspec);
-	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Resent-From", write_addrspec);
-	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Resent-To", write_addrspec);
-	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Resent-Cc", write_addrspec);
-	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Resent-Bcc", write_addrspec);
-	
-	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Subject", write_subject);
 	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Received", write_received);
 	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Message-Id", write_msgid);
-	g_mime_header_register_writer (((GMimeObject *) message)->headers, "References", write_addrspec);
 }
 
 static gboolean
@@ -198,7 +182,7 @@ typedef void (*token_skip_t) (const char **in);
 
 struct _received_token {
 	char *token;
-	size_t len;
+	int len;
 	token_skip_t skip;
 };
 
@@ -206,7 +190,7 @@ extern void decode_lwsp   (const char **in);
 
 static void skip_atom     (const char **in);
 static void skip_domain   (const char **in);
-static void skip_addr     (const char **in);
+static void skip_addrspec (const char **in);
 static void skip_msgid    (const char **in);
 
 static struct _received_token received_tokens[] = {
@@ -215,9 +199,11 @@ static struct _received_token received_tokens[] = {
 	{ "via ",  4, skip_atom     },
 	{ "with ", 5, skip_atom     },
 	{ "id ",   3, skip_msgid    },
-	{ "for ",  4, skip_addr     }
+	{ "for ",  4, skip_addrspec }
 };
 
+#define NUM_RECEIVED_TOKENS (sizeof (received_tokens) / sizeof (received_tokens[0]))
+
 static void
 skip_atom (const char **in)
 {
@@ -230,29 +216,6 @@ skip_atom (const char **in)
 	*in = inptr;
 }
 
-static void
-skip_comment (const char **in)
-{
-	register const char *inptr = *in;
-	int depth = 1;
-	
-	if (*inptr == '(')
-		inptr++;
-	
-	while (*inptr && depth > 0) {
-		if (*inptr == '(')
-			depth++;
-		else if (*inptr == ')')
-			depth--;
-		inptr++;
-	}
-	
-	if (*inptr == ')')
-		inptr++;
-	
-	*in = inptr;
-}
-
 static void
 skip_quoted_string (const char **in)
 {
@@ -373,24 +336,6 @@ skip_addrspec (const char **in)
 	*in = inptr;
 }
 
-static void
-skip_addr (const char **in)
-{
-	const char *inptr = *in;
-	
-	decode_lwsp (&inptr);
-	if (*inptr == '<') {
-		inptr++;
-		skip_addrspec (&inptr);
-		if (*inptr == '>')
-			inptr++;
-	} else {
-		skip_addrspec (&inptr);
-	}
-	
-	*in = inptr;
-}
-
 static void
 skip_msgid (const char **in)
 {
@@ -399,126 +344,81 @@ skip_msgid (const char **in)
 	decode_lwsp (&inptr);
 	if (*inptr == '<') {
 		inptr++;
+		decode_lwsp (&inptr);
 		skip_addrspec (&inptr);
 		if (*inptr == '>')
 			inptr++;
-	} else {
-		skip_atom (&inptr);
 	}
 	
 	*in = inptr;
 }
 
 
-struct _received_part {
-	struct _received_part *next;
-	const char *start;
-	size_t len;
-};
-
 static ssize_t
 write_received (GMimeStream *stream, const char *name, const char *value)
 {
-	struct _received_part *parts, *part, *tail;
-	const char *lwsp, *inptr;
+	const char *start, *inptr;
 	ssize_t nwritten;
 	GString *str;
 	int len, i;
 	
-	while (is_lwsp (*value))
-		value++;
-	
-	if (*value == '\0')
-		return 0;
-	
 	str = g_string_new (name);
 	g_string_append_len (str, ": ", 2);
 	len = 10;
 	
-	tail = parts = part = g_alloca (sizeof (struct _received_part));
-	part->start = inptr = value;
-	part->next = NULL;
-	
+	start = inptr = value;
 	while (*inptr) {
-		for (i = 0; i < G_N_ELEMENTS (received_tokens); i++) {
+		while (is_lwsp (*inptr))
+			inptr++;
+		
+		for (i = 0; i < NUM_RECEIVED_TOKENS; i++) {
 			if (!strncmp (inptr, received_tokens[i].token, received_tokens[i].len)) {
-				if (inptr > part->start) {
-					part->len = lwsp - part->start;
-					
-					part = g_alloca (sizeof (struct _received_part));
-					part->start = inptr;
-					part->next = NULL;
-					
-					tail->next = part;
-					tail = part;
+				if ((inptr - start) + len > GMIME_FOLD_LEN && start != value) {
+					g_string_append (str, "\n\t");
+					while (is_lwsp (*start))
+						start++;
+					len = 1;
 				}
 				
+				/* write the last section */
+				g_string_append_len (str, start, inptr - start);
+				len += (inptr - start);
+				start = inptr;
+				
 				inptr += received_tokens[i].len;
 				received_tokens[i].skip (&inptr);
-				
-				lwsp = inptr;
-				while (is_lwsp (*inptr))
-					inptr++;
-				
-				if (*inptr == ';') {
-					inptr++;
-					
-					part->len = inptr - part->start;
-					
-					lwsp = inptr;
-					while (is_lwsp (*inptr))
-						inptr++;
-					
-					part = g_alloca (sizeof (struct _received_part));
-					part->start = inptr;
-					part->next = NULL;
-					
-					tail->next = part;
-					tail = part;
-				}
+				decode_lwsp (&inptr);
+				inptr--;
 				
 				break;
 			}
 		}
 		
-		if (i == G_N_ELEMENTS (received_tokens)) {
-			while (*inptr && !is_lwsp (*inptr))
-				inptr++;
+		if (*inptr == ';') {
+			if ((inptr - start) + len > GMIME_FOLD_LEN && start != value) {
+				g_string_append (str, "\n\t");
+				while (is_lwsp (*start))
+					start++;
+				len = 1;
+			}
 			
-			lwsp = inptr;
-			while (is_lwsp (*inptr))
-				inptr++;
+			inptr++;
+			g_string_append_len (str, start, inptr - start);
+			len += (inptr - start);
+			start = inptr;
 		}
 		
-		if (*inptr == '(') {
-			skip_comment (&inptr);
-			
-			lwsp = inptr;
-			while (is_lwsp (*inptr))
-				inptr++;
-		}
+		inptr++;
 	}
 	
-	part->len = lwsp - part->start;
-	
-	lwsp = NULL;
-	part = parts;
-	do {
-		len += lwsp ? part->start - lwsp : 0;
-		if (len + part->len > GMIME_FOLD_LEN && part != parts) {
-			g_string_append (str, "\n\t");
-			len = 1;
-		} else if (lwsp) {
-			g_string_append_len (str, lwsp, part->start - lwsp);
-		}
-		
-		g_string_append_len (str, part->start, part->len);
-		lwsp = part->start + part->len;
-		len += part->len;
-		
-		part = part->next;
-	} while (part != NULL);
+	if ((inptr - start) + len > GMIME_FOLD_LEN && start != value) {
+		g_string_append (str, "\n\t");
+		while (is_lwsp (*start))
+			start++;
+		len = 1;
+	}
 	
+	g_string_append_len (str, start, inptr - start);
 	g_string_append_c (str, '\n');
 	
 	nwritten = g_mime_stream_write (stream, str->str, str->len);
@@ -527,22 +427,6 @@ write_received (GMimeStream *stream, const char *name, const char *value)
 	return nwritten;
 }
 
-static ssize_t
-write_subject (GMimeStream *stream, const char *name, const char *value)
-{
-	char *unfolded, *folded;
-	ssize_t n;
-	
-	unfolded = g_strdup_printf ("%s: %s\n", name, value);
-	folded = g_mime_utils_unstructured_header_fold (unfolded);
-	g_free (unfolded);
-	
-	n = g_mime_stream_write_string (stream, folded);
-	g_free (folded);
-	
-	return n;
-}
-
 static ssize_t
 write_msgid (GMimeStream *stream, const char *name, const char *value)
 {
@@ -551,22 +435,6 @@ write_msgid (GMimeStream *stream, const char *name, const char *value)
 	return g_mime_stream_printf (stream, "%s: %s\n", name, value);
 }
 
-static ssize_t
-write_addrspec (GMimeStream *stream, const char *name, const char *value)
-{
-	char *unfolded, *folded;
-	ssize_t n;
-	
-	unfolded = g_strdup_printf ("%s: %s\n", name, value);
-	folded = g_mime_utils_structured_header_fold (unfolded);
-	g_free (unfolded);
-	
-	n = g_mime_stream_write_string (stream, folded);
-	g_free (folded);
-	
-	return n;
-}
-
 
 enum {
 	HEADER_FROM,
@@ -657,9 +525,6 @@ message_add_header (GMimeObject *object, const char *header, const char *value)
 		else
 			GMIME_OBJECT_CLASS (parent_class)->add_header (object, header, value);
 	}
-	
-	if (((GMimeMessage *) object)->mime_part)
-		g_mime_header_set_raw (((GMimeMessage *) object)->mime_part->headers, NULL);
 }
 
 static void
@@ -677,9 +542,6 @@ message_set_header (GMimeObject *object, const char *header, const char *value)
 		else
 			GMIME_OBJECT_CLASS (parent_class)->set_header (object, header, value);
 	}
-	
-	if (((GMimeMessage *) object)->mime_part)
-		g_mime_header_set_raw (((GMimeMessage *) object)->mime_part->headers, NULL);
 }
 
 static const char *
@@ -759,9 +621,6 @@ message_remove_header (GMimeObject *object, const char *header)
 	}
 	
 	GMIME_OBJECT_CLASS (parent_class)->remove_header (object, header);
-	
-	if (((GMimeMessage *) object)->mime_part)
-		g_mime_header_set_raw (((GMimeMessage *) object)->mime_part->headers, NULL);
 }
 
 
@@ -777,15 +636,10 @@ message_get_headers (GMimeObject *object)
 	stream = g_mime_stream_mem_new ();
 	g_mime_stream_mem_set_byte_array (GMIME_STREAM_MEM (stream), ba);
 	
-	if (message->mime_part && g_mime_header_has_raw (message->mime_part->headers)) {
-		/* if the mime part has raw headers, then it contains the message headers as well */
+	g_mime_header_write_to_stream (object->headers, stream);
+	if (message->mime_part) {
+		g_mime_stream_write_string (stream, "MIME-Version: 1.0\n");
 		g_mime_header_write_to_stream (message->mime_part->headers, stream);
-	} else {
-		g_mime_header_write_to_stream (object->headers, stream);
-		if (message->mime_part) {
-			g_mime_stream_write_string (stream, "MIME-Version: 1.0\n");
-			g_mime_header_write_to_stream (message->mime_part->headers, stream);
-		}
 	}
 	
 	g_object_unref (stream);
@@ -802,25 +656,24 @@ message_write_to_stream (GMimeObject *object, GMimeStream *stream)
 	GMimeMessage *message = (GMimeMessage *) object;
 	ssize_t nwritten, total = 0;
 	
-	if (!(message->mime_part && g_mime_header_has_raw (message->mime_part->headers))) {
-		/* write the content headers */
-		if ((nwritten = g_mime_header_write_to_stream (object->headers, stream)) == -1)
+	/* write the content headers */
+	nwritten = g_mime_header_write_to_stream (object->headers, stream);
+	if (nwritten == -1)
+		return -1;
+	
+	total += nwritten;
+	
+	if (message->mime_part) {
+		nwritten = g_mime_stream_write_string (stream, "MIME-Version: 1.0\n");
+		if (nwritten == -1)
 			return -1;
 		
 		total += nwritten;
 		
-		if (message->mime_part) {
-			if ((nwritten = g_mime_stream_write_string (stream, "MIME-Version: 1.0\n")) == -1)
-				return -1;
-			
-			total += nwritten;
-		}
-	}
-	
-	if (message->mime_part) {
 		nwritten = g_mime_object_write_to_stream (message->mime_part, stream);
 	} else {
-		if ((nwritten = g_mime_stream_write (stream, "\n", 1)) == -1)
+		nwritten = g_mime_stream_write (stream, "\n", 1);
+		if (nwritten == -1)
 			return -1;
 	}
 	
@@ -875,25 +728,16 @@ message_set_sender (GMimeMessage *message, const char *sender)
  * @sender: The name and address of the sender
  *
  * Set the sender's name and address on the MIME Message.
- * (ex: "\"Joe Sixpack\" &lt;joe@sixpack.org&gt;")
+ * (ex: "\"Joe Sixpack\" <joe@sixpack.org>")
  **/
 void
 g_mime_message_set_sender (GMimeMessage *message, const char *sender)
 {
-	InternetAddressList *addrlist;
-	char *encoded;
-	
 	g_return_if_fail (GMIME_IS_MESSAGE (message));
 	g_return_if_fail (sender != NULL);
 	
 	message_set_sender (message, sender);
-	
-	addrlist = internet_address_parse_string (sender);
-	encoded = internet_address_list_to_string (addrlist, TRUE);
-	internet_address_list_destroy (addrlist);
-	
-	g_mime_header_set (GMIME_OBJECT (message)->headers, "From", encoded);
-	g_free (encoded);
+	g_mime_header_set (GMIME_OBJECT (message)->headers, "From", message->from);
 }
 
 
@@ -1298,28 +1142,6 @@ g_mime_message_get_header (GMimeMessage *message, const char *header)
 }
 
 
-/**
- * g_mime_message_get_mime_part:
- * @message: MIME Message
- *
- * Gets the toplevel MIME part contained within @message.
- *
- * Returns the toplevel MIME part of @message.
- **/
-GMimeObject *
-g_mime_message_get_mime_part (GMimeMessage *message)
-{
-	g_return_val_if_fail (GMIME_IS_MESSAGE (message), NULL);
-	
-	if (message->mime_part == NULL)
-		return NULL;
-	
-	g_object_ref (message->mime_part);
-	
-	return message->mime_part;
-}
-
-
 /**
  * g_mime_message_set_mime_part:
  * @message: MIME Message
@@ -1333,7 +1155,6 @@ g_mime_message_set_mime_part (GMimeMessage *message, GMimeObject *mime_part)
 	g_return_if_fail (GMIME_IS_MESSAGE (message));
 	
 	g_object_ref (mime_part);
-	g_mime_header_set_raw (mime_part->headers, NULL);
 	
 	if (message->mime_part)
 		g_object_unref (message->mime_part);
@@ -1350,7 +1171,7 @@ g_mime_message_set_mime_part (GMimeMessage *message, GMimeObject *mime_part)
  * Write the contents of the MIME Message to @stream.
  *
  * WARNING: This interface is deprecated. Use
- * g_mime_object_write_to_stream() instead.
+ * #g_mime_object_write_to_stream instead.
  *
  * Returns -1 on fail.
  **/
@@ -1371,7 +1192,7 @@ g_mime_message_write_to_stream (GMimeMessage *message, GMimeStream *stream)
  * Allocates a string buffer containing the mime message @message.
  *
  * WARNING: This interface is deprecated. Use
- * g_mime_object_to_string() instead.
+ * #g_mime_object_to_string instead.
  *
  * Returns an allocated string containing the MIME Message.
  **/
@@ -1384,7 +1205,8 @@ g_mime_message_to_string (GMimeMessage *message)
 }
 
 
-/* The proper way to handle a multipart/alternative part is to return
+/**
+ * The proper way to handle a multipart/alternative part is to return
  * the last part that we know how to render. For our purposes, we are
  * going to assume:
  *
@@ -1447,7 +1269,7 @@ handle_multipart_mixed (GMimeMultipart *multipart, gboolean want_plain, gboolean
 		} else if (g_mime_content_type_is_type (type, "text", "*")) {
 			if (!strcasecmp (type->subtype, want_plain ? "plain" : "html")) {
 				/* we got what we came for */
-				*is_html = !strcasecmp (type->subtype, "html");
+				*is_html = !want_plain;
 				return mime_part;
 			}
 			
@@ -1501,7 +1323,7 @@ g_mime_message_get_body (const GMimeMessage *message, gboolean want_plain, gbool
 	
 	type = g_mime_object_get_content_type (message->mime_part);
 	if (GMIME_IS_MULTIPART (message->mime_part)) {
-		/* let's see if we can find a body in the multipart */
+		/* lets see if we can find a body in the multipart */
 		multipart = GMIME_MULTIPART (message->mime_part);
 		if (g_mime_content_type_is_type (type, "multipart", "alternative"))
 			mime_part = handle_multipart_alternative (multipart, want_plain, is_html);
@@ -1509,10 +1331,10 @@ g_mime_message_get_body (const GMimeMessage *message, gboolean want_plain, gbool
 			mime_part = handle_multipart_mixed (multipart, want_plain, is_html);
 	} else if (g_mime_content_type_is_type (type, "text", "*")) {
 		/* this *has* to be the message body */
-		if (g_mime_content_type_is_type (type, "text", "html"))
-			*is_html = TRUE;
+		if (g_mime_content_type_is_type (type, "text", want_plain ? "plain" : "html"))
+			*is_html = !want_plain;
 		else
-			*is_html = FALSE;
+			*is_html = want_plain;
 		
 		mime_part = message->mime_part;
 	}
