@@ -25,7 +25,6 @@
 #include <config.h>
 #endif
 
-#include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
 #include <regex.h>
@@ -100,11 +99,6 @@ struct _GMimeParserPrivate {
 	unsigned char *headerptr;
 	unsigned int headerleft;
 	
-	/* raw header buffer */
-	unsigned char *rawbuf;
-	unsigned char *rawptr;
-	unsigned int rawleft;
-	
 	off_t headers_start;
 	off_t header_start;
 	
@@ -125,19 +119,18 @@ struct _GMimeParserPrivate {
 struct _boundary_stack {
 	struct _boundary_stack *parent;
 	unsigned char *boundary;
-	size_t boundarylen;
-	size_t boundarylenfinal;
-	size_t boundarylenmax;
+	unsigned int boundarylen;
+	unsigned int boundarylenfinal;
+	unsigned int boundarylenmax;
 	off_t content_end;
 };
 
-
 static void
 parser_push_boundary (GMimeParser *parser, const char *boundary)
 {
 	struct _GMimeParserPrivate *priv = parser->priv;
 	struct _boundary_stack *s;
-	size_t max;
+	unsigned int max;
 	
 	max = priv->bounds ? priv->bounds->boundarylenmax : 0;
 	
@@ -265,7 +258,6 @@ g_mime_parser_init (GMimeParser *parser, GMimeParserClass *klass)
 	parser->priv->scan_from = FALSE;
 	parser->priv->have_regex = FALSE;
 	parser->priv->persist_stream = TRUE;
-	parser->priv->respect_content_length = FALSE;
 }
 
 static void
@@ -312,10 +304,6 @@ parser_init (GMimeParser *parser, GMimeStream *stream)
 	priv->headerptr = priv->headerbuf;
 	priv->headerleft = SCAN_HEAD;
 	
-	priv->rawbuf = g_malloc (SCAN_HEAD + 1);
-	priv->rawptr = priv->rawbuf;
-	priv->rawleft = SCAN_HEAD;
-	
 	priv->headers_start = -1;
 	priv->header_start = -1;
 	
@@ -338,8 +326,8 @@ parser_close (GMimeParser *parser)
 	
 	g_byte_array_free (priv->from_line, TRUE);
 	
-	g_free (priv->headerbuf);
-	g_free (priv->rawbuf);
+	if (priv->headerbuf)
+		g_free (priv->headerbuf);
 	
 	header_raw_clear (&priv->headers);
 	
@@ -368,7 +356,7 @@ g_mime_parser_new (void)
 {
 	GMimeParser *parser;
 	
-	parser = g_object_new (GMIME_TYPE_PARSER, NULL);
+	parser = g_object_new (GMIME_TYPE_PARSER, NULL, NULL);
 	
 	return parser;
 }
@@ -409,11 +397,11 @@ g_mime_parser_new_with_stream (GMimeStream *stream)
  * or resetting of the stream. Anything that will/could change the
  * current stream's offset is PROHIBITED.
  *
- * It is also recommended that you not use g_mime_stream_tell()
- * because it will not necessarily give you the current @parser offset
- * since @parser handles its own internal read-ahead buffer. Instead,
- * it is recommended that you use g_mime_parser_tell() if you have a
- * reason to need the current offset of the @parser.
+ * It is also recommended that you not use #g_mime_stream_tell because
+ * it will not necessarily give you the current @parser offset since
+ * @parser handles its own internal read-ahead buffer. Instead, it is
+ * recommended that you use #g_mime_parser_tell if you have a reason
+ * to need the current offset of the @parser.
  **/
 void
 g_mime_parser_init_with_stream (GMimeParser *parser, GMimeStream *stream)
@@ -586,8 +574,8 @@ parser_fill (GMimeParser *parser)
 {
 	struct _GMimeParserPrivate *priv = parser->priv;
 	unsigned char *inbuf, *inptr, *inend;
+	size_t inlen, atleast = SCAN_HEAD;
 	ssize_t nread;
-	size_t inlen;
 	
 	inbuf = priv->inbuf;
 	inptr = priv->inptr;
@@ -596,6 +584,11 @@ parser_fill (GMimeParser *parser)
 	
 	g_assert (inptr <= inend);
 	
+	atleast = MAX (atleast, priv->bounds ? priv->bounds->boundarylenmax : 0);
+	
+	if (inlen > atleast)
+		return inlen;
+	
 	/* attempt to align 'inend' with realbuf + SCAN_HEAD */
 	if (inptr >= inbuf) {
 		inbuf -= inlen < SCAN_HEAD ? inlen : SCAN_HEAD;
@@ -618,7 +611,8 @@ parser_fill (GMimeParser *parser)
 	priv->inend = inbuf;
 	inend = priv->realbuf + SCAN_HEAD + SCAN_BUF - 1;
 	
-	if ((nread = g_mime_stream_read (priv->stream, inbuf, inend - inbuf)) > 0)
+	nread = g_mime_stream_read (priv->stream, inbuf, inend - inbuf);
+	if (nread > 0)
 		priv->inend += nread;
 	
 	priv->offset = g_mime_stream_tell (priv->stream);
@@ -630,9 +624,6 @@ parser_fill (GMimeParser *parser)
 static off_t
 parser_offset (struct _GMimeParserPrivate *priv, unsigned char *inptr)
 {
-	if (priv->offset == -1)
-		return -1;
-	
 	if (!inptr)
 		inptr = priv->inptr;
 	
@@ -734,9 +725,9 @@ parser_step_from (GMimeParser *parser)
 	return 0;
 }
 
-#define header_append(priv, start, len) G_STMT_START {                    \
+#define header_backup(priv, start, len) G_STMT_START {                    \
 	if (priv->headerleft <= len) {                                    \
-		size_t hlen, hoff;                                        \
+		unsigned int hlen, hoff;                                  \
 		                                                          \
 		hlen = hoff = priv->headerptr - priv->headerbuf;          \
 		hlen = hlen ? hlen : 1;                                   \
@@ -754,35 +745,10 @@ parser_step_from (GMimeParser *parser)
 	priv->headerleft -= len;                                          \
 } G_STMT_END
 
-#define raw_header_append(priv, start, len) G_STMT_START {                \
-	if (priv->rawleft <= len) {                                       \
-		size_t hlen, hoff;                                        \
-		                                                          \
-		hlen = hoff = priv->rawptr - priv->rawbuf;                \
-		hlen = hlen ? hlen : 1;                                   \
-		                                                          \
-		while (hlen < hoff + len)                                 \
-			hlen <<= 1;                                       \
-		                                                          \
-		priv->rawbuf = g_realloc (priv->rawbuf, hlen + 1);        \
-		priv->rawptr = priv->rawbuf + hoff;                       \
-		priv->rawleft = hlen - hoff;                              \
-	}                                                                 \
-	                                                                  \
-	memcpy (priv->rawptr, start, len);                                \
-	priv->rawptr += len;                                              \
-	priv->rawleft -= len;                                             \
-} G_STMT_END
-
-#define raw_header_reset(priv) G_STMT_START {                             \
-	priv->rawleft += priv->rawptr - priv->rawbuf;                     \
-	priv->rawptr = priv->rawbuf;                                      \
-} G_STMT_END
-
 #define header_parse(parser, priv, hend) G_STMT_START {                   \
 	register unsigned char *colon;                                    \
 	struct _header_raw *header;                                       \
-	size_t hlen;                                                      \
+	unsigned int hlen;                                                \
 	                                                                  \
 	header = g_new (struct _header_raw, 1);                           \
 	header->next = NULL;                                              \
@@ -853,29 +819,23 @@ parser_step_headers (GMimeParser *parser)
 			while (*inptr != '\n')
 				inptr++;
 			
-			raw_header_append (priv, start, inptr - start);
-			
 			if (inptr == inend) {
 				/* we don't have enough data to tell if we
 				   got all of the header or not... */
-				priv->inptr = inptr;
-				len = inend - inptr;
+				priv->inptr = start;
+				len = inend - start;
 				goto refill;
 			}
 			
 			/* check to see if we've reached the end of the headers */
-			if (!priv->midline && (inptr == start || (inptr - start == 1 && *start == '\r')))
+			if (!priv->midline && inptr == start)
 				goto headers_end;
 			
 			len = inptr - start;
-			if (inptr[-1] == '\r')
-				len--;
-			
-			header_append (priv, start, len);
+			header_backup (priv, start, len);
 			
 			if (inptr < inend) {
 				/* inptr has to be less than inend - 1 */
-				raw_header_append (priv, inptr, 1);
 				inptr++;
 				if (*inptr == ' ' || *inptr == '\t') {
 					priv->midline = TRUE;
@@ -896,15 +856,13 @@ parser_step_headers (GMimeParser *parser)
 	inptr = priv->inptr;
 	inend = priv->inend;
 	
-	header_append (priv, inptr, inend - inptr);
+	header_backup (priv, inptr, inend - inptr);
 	
  headers_end:
 	
 	if (priv->headerptr > priv->headerbuf)
 		header_parse (parser, priv, hend);
 	
-	*priv->rawptr = '\0';
-	
 	priv->state = GMIME_PARSER_STATE_HEADERS_END;
 	
 	g_assert (inptr <= priv->inend);
@@ -920,7 +878,8 @@ parser_content_type (GMimeParser *parser)
 	struct _GMimeParserPrivate *priv = parser->priv;
 	const char *content_type;
 	
-	if ((content_type = header_raw_find (priv->headers, "Content-Type", NULL)))
+	content_type = header_raw_find (priv->headers, "Content-Type", NULL);
+	if (content_type)
 		return g_mime_content_type_new_from_string (content_type);
 	
 	return NULL;
@@ -1007,13 +966,10 @@ enum {
 			  (len >= 2 && (start[0] == '-' && start[1] == '-')))
 
 static int
-check_boundary (struct _GMimeParserPrivate *priv, const unsigned char *start, size_t len)
+check_boundary (struct _GMimeParserPrivate *priv, const unsigned char *start, int len)
 {
 	off_t offset = parser_offset (priv, (unsigned char *) start);
 	
-	if (start[len - 1] == '\r')
-		len--;
-	
 	if (possible_boundary (priv->scan_from, start, len)) {
 		struct _boundary_stack *s;
 		
@@ -1058,7 +1014,7 @@ check_boundary (struct _GMimeParserPrivate *priv, const unsigned char *start, si
  **/
 
 static int
-parser_scan_content (GMimeParser *parser, GByteArray *content, int *crlf)
+parser_scan_content (GMimeParser *parser, GByteArray *content)
 {
 	struct _GMimeParserPrivate *priv = parser->priv;
 	register unsigned char *inptr;
@@ -1102,10 +1058,9 @@ parser_scan_content (GMimeParser *parser, GByteArray *content, int *crlf)
 			len = inptr - start;
 			
 			if (inptr < inend) {
+				inptr++;
 				if ((found = check_boundary (priv, start, len)))
 					goto boundary;
-				
-				inptr++;
 				len++;
 			} else {
 				/* didn't find an end-of-line */
@@ -1134,15 +1089,6 @@ parser_scan_content (GMimeParser *parser, GByteArray *content, int *crlf)
 	/* don't chew up the boundary */
 	priv->inptr = start;
 	
-	if (found != FOUND_EOS) {
-		if (inptr[-1] == '\r')
-			*crlf = 2;
-		else
-			*crlf = 1;
-	} else {
-		*crlf = 0;
-	}
-	
 	return found;
 }
 
@@ -1155,20 +1101,19 @@ parser_scan_mime_part_content (GMimeParser *parser, GMimePart *mime_part, int *f
 	GMimeDataWrapper *wrapper;
 	GMimeStream *stream;
 	off_t start, end;
-	int crlf;
 	
 	if (priv->persist_stream && priv->seekable)
 		start = parser_offset (priv, NULL);
 	else
 		content = g_byte_array_new ();
 	
-	*found = parser_scan_content (parser, content, &crlf);
+	*found = parser_scan_content (parser, content);
 	if (*found != FOUND_EOS) {
 		/* last '\n' belongs to the boundary */
 		if (priv->persist_stream && priv->seekable)
-			end = parser_offset (priv, NULL) - crlf;
+			end = parser_offset (priv, NULL) - 1;
 		else
-			g_byte_array_set_size (content, MAX (content->len - crlf, 0));
+			g_byte_array_set_size (content, MAX (content->len - 1, 0));
 	} else if (priv->persist_stream && priv->seekable) {
 		end = parser_offset (priv, NULL);
 	}
@@ -1208,17 +1153,19 @@ parser_scan_message_part (GMimeParser *parser, GMimeMessagePart *mpart, int *fou
 		header = header->next;
 	}
 	
-	if (!(content_type = parser_content_type (parser)))
+	content_type = parser_content_type (parser);
+	if (!content_type)
 		content_type = g_mime_content_type_new ("text", "plain");
 	
 	parser_unstep (parser);
-	if (g_mime_content_type_is_type (content_type, "multipart", "*")) {
+	if (content_type && g_mime_content_type_is_type (content_type, "multipart", "*")) {
 		object = parser_construct_multipart (parser, content_type, found);
 	} else {
 		object = parser_construct_leaf_part (parser, content_type, found);
 	}
 	
-	message->mime_part = object;
+	g_mime_message_set_mime_part (message, object);
+	g_object_unref (object);
 	
 	g_mime_message_part_set_message (mpart, message);
 	g_object_unref (message);
@@ -1235,6 +1182,12 @@ parser_construct_leaf_part (GMimeParser *parser, GMimeContentType *content_type,
 	while (parser_step (parser) != GMIME_PARSER_STATE_HEADERS_END)
 		;
 	
+	if (!content_type) {
+		content_type = parser_content_type (parser);
+		if (!content_type)
+			content_type = g_mime_content_type_new ("text", "plain");
+	}
+	
 	object = g_mime_object_new_type (content_type->type, content_type->subtype);
 	header = priv->headers;
 	while (header) {
@@ -1244,12 +1197,7 @@ parser_construct_leaf_part (GMimeParser *parser, GMimeContentType *content_type,
 	
 	header_raw_clear (&priv->headers);
 	
-	if (object->content_type)
-		g_mime_content_type_destroy (object->content_type);
-	object->content_type = content_type;
-	
-	g_mime_header_set_raw (object->headers, priv->rawbuf);
-	raw_header_reset (priv);
+	g_mime_object_set_content_type (object, content_type);
 	
 	/* skip empty line after headers */
 	parser_skip_line (parser);
@@ -1263,53 +1211,27 @@ parser_construct_leaf_part (GMimeParser *parser, GMimeContentType *content_type,
 	return object;
 }
 
-static void
-crlf2lf (unsigned char *in)
-{
-	register unsigned char *inptr = in;
-	register unsigned char *outptr;
-	
-	while (*inptr != '\0' && !(inptr[0] == '\r' && inptr[1] == '\n'))
-		inptr++;
-	
-	if (*inptr == '\0')
-		return;
-	
-	outptr = inptr++;
-	
-	while (*inptr != '\0') {
-		while (*inptr != '\0' && !(inptr[0] == '\r' && inptr[1] == '\n'))
-			*outptr++ = *inptr++;
-		
-		if (*inptr == '\r')
-			inptr++;
-	}
-	
-	*outptr = '\0';
-}
-
 static int
 parser_scan_multipart_face (GMimeParser *parser, GMimeMultipart *multipart, gboolean preface)
 {
 	GByteArray *buffer;
 	const char *face;
-	int crlf, found;
+	int found;
 	
 	buffer = g_byte_array_new ();
-	found = parser_scan_content (parser, buffer, &crlf);
+	found = parser_scan_content (parser, buffer);
 	
-	if (buffer->len >= crlf) {
-		/* last '\n' belongs to the boundary */
-		g_byte_array_set_size (buffer, buffer->len + 1);
-		buffer->data[buffer->len - crlf - 1] = '\0';
-		crlf2lf (buffer->data);
+	/* last '\n' belongs to the boundary */
+	if (buffer->len) {
+		buffer->data[buffer->len - 1] = '\0';
 		face = buffer->data;
-		
-		if (preface)
-			g_mime_multipart_set_preface (multipart, face);
-		else
-			g_mime_multipart_set_postface (multipart, face);
-	}
+	} else
+		face = NULL;
+	
+	if (preface)
+		g_mime_multipart_set_preface (multipart, face);
+	else
+		g_mime_multipart_set_postface (multipart, face);
 	
 	g_byte_array_free (buffer, TRUE);
 	
@@ -1333,7 +1255,8 @@ parser_scan_multipart_subparts (GMimeParser *parser, GMimeMultipart *multipart)
 		/* get the headers */
 		parser_step_headers (parser);
 		
-		if (!(content_type = parser_content_type (parser)))
+		content_type = parser_content_type (parser);
+		if (!content_type)
 			content_type = g_mime_content_type_new ("text", "plain");
 		
 		parser_unstep (parser);
@@ -1380,12 +1303,7 @@ parser_construct_multipart (GMimeParser *parser, GMimeContentType *content_type,
 	
 	header_raw_clear (&priv->headers);
 	
-	if (object->content_type)
-		g_mime_content_type_destroy (object->content_type);
-	object->content_type = content_type;
-	
-	g_mime_header_set_raw (object->headers, priv->rawbuf);
-	raw_header_reset (priv);
+	g_mime_object_set_content_type (object, content_type);
 	
 	multipart = (GMimeMultipart *) object;
 	
@@ -1429,7 +1347,8 @@ parser_construct_part (GMimeParser *parser)
 	while (parser_step (parser) != GMIME_PARSER_STATE_HEADERS_END)
 		;
 	
-	if (!(content_type = parser_content_type (parser)))
+	content_type = parser_content_type (parser);
+	if (!content_type)
 		content_type = g_mime_content_type_new ("text", "plain");
 	
 	parser_unstep (parser);
@@ -1491,7 +1410,8 @@ parser_construct_message (GMimeParser *parser)
 			priv->bounds->content_end = parser_offset (priv, NULL) + content_length;
 	}
 	
-	if (!(content_type = parser_content_type (parser)))
+	content_type = parser_content_type (parser);
+	if (!content_type)
 		content_type = g_mime_content_type_new ("text", "plain");
 	
 	parser_unstep (parser);
@@ -1501,7 +1421,8 @@ parser_construct_message (GMimeParser *parser)
 		object = parser_construct_leaf_part (parser, content_type, &found);
 	}
 	
-	message->mime_part = object;
+	g_mime_message_set_mime_part (message, object);
+	g_object_unref (object);
 	
 	if (priv->scan_from) {
 		priv->state = GMIME_PARSER_STATE_FROM;
@@ -1534,7 +1455,7 @@ g_mime_parser_construct_message (GMimeParser *parser)
  * @parser: MIME parser object
  *
  * Gets the mbox-style From-line of the most recently parsed message
- * (gotten from g_mime_parser_construct_message()).
+ * (gotten from #g_mime_parser_construct_message).
  *
  * Returns the mbox-style From-line of the most recently parsed
  * message or %NULL on error.
@@ -1562,7 +1483,7 @@ g_mime_parser_get_from (GMimeParser *parser)
  * @parser: MIME parser object
  *
  * Gets the offset of the most recently parsed mbox-style From-line
- * (gotten from g_mime_parser_construct_message()).
+ * (gotten from #g_mime_parser_construct_message).
  *
  * Returns the offset of the most recently parsed mbox-style From-line
  * or -1 on error.
